%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% This file does the time series predictability analysis
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clearvars; clc

%Here you pre-define the sample that you want to use
year0=1948; quarter0=year0*10+1; month0=year0*100+01; %Starting year/quarter/month
year1=2014; quarter1=year1*10+4; month1=year1*100+12; %Ending year/quarter/month

% A description of data sets
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data1: FF3 factors annual
% Data2: Shiller annual data:http://www.econ.yale.edu/~shiller/data.htm long term stock, bond, interest rate and consumption data 
% Data4: Baa bond yield: http://research.stlouisfed.org/fred2/series/BAA/downloaddata?cid=119. Maturity is 30 years  
% Data5: NIPA table 1.1.5: contains nominal GDP
% Data6: NIPA table 1.1.9: contains implicit price of GDP
% Data7: NIPA table 2.3.4: contains Price Indexes for Personal Consumption Expenditures
% Data9: NIPA table 6.4: contains number of full and part time employees
% Data10: NIPA table6.6: contains wage and salary Accruals Per Full-Time Equivalent Employee
% Data11: NIPA table6.2: contains compensatoin by employees
% Data15: contains market value of equity, book value of debt from flow of funds
% Data19: FF3Factor_Monthly.txt:             contains FF3 monthly factors
% Data24: final_bankcruptcy: contains historical default rate
% Data32: NIPATable7.11: contains the annual data from NIPA on interests

% Load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Data1 = load('FF3Factor_Annually.txt');                   ini1=find(Data1(:,1)==year0); fin1=find(Data1(:,1)==year1); 
Data2 = load('ShillerAnnual.txt');                        ini2=find(Data2(:,1)==year0); fin2=find(Data2(:,1)==year1);                 
Data4 = load('BAA.txt');                                  ini4=find(Data4(:,1)==year0); fin4=find(Data4(:,1)==year1);
Data5 = load('NIPATable1.1.5.txt');                       ini5=find(Data5(:,1)==year0); fin5=find(Data5(:,1)==year1);
Data6 = load('NIPATable1.1.9.txt');                       ini6=find(Data6(:,1)==year0); fin6=find(Data6(:,1)==year1);
Data7 = load('NIPATable2.3.4.txt');                       ini7=find(Data7(:,1)==year0); fin7=find(Data7(:,1)==year1);
Data9 = load('NIPATable6.4.txt');                         ini9=find(Data9(:,1)==year0); fin9=find(Data9(:,1)==year1);
Data10 = load('NIPATable6.6.txt');                        ini10=find(Data10(:,1)==year0); fin10=find(Data10(:,1)==year1);
Data11 = load('NIPATable6.2.txt');                        ini11=find(Data11(:,1)==year0); fin11=find(Data11(:,1)==year1);
Data15 = load('AnnualEquityDebtFlowFund.txt');            ini15=find(Data15(:,1)==year0); fin15=find(Data15(:,1)==year1); % starts in 1946
Data19 = load('FF3Factor_Monthly.txt');                   ini19=find(Data19(:,1)==month0);fin19=find(Data19(:,1)==month1); 
Data24 = load('final_bankruptcy.txt');                    ini24=find(Data24(:,1)==year0); fin24=find(Data24(:,1)==year1);  % default rates stop at 2006
Data32 = load('NIPATable7.11.txt');                       ini32=find(Data32(:,1)==year0); fin32=find(Data32(:,1)==year1);

% Calculate summary stats
% inflation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Year  = Data1(:,1);
Cpi   = Data7(:,2);                             % CPI index
Inf   = zeros(length(Cpi),1);
Inf(2:end)  = Cpi(2:end)./Cpi(1:end-1);         % annual inflation
Inf   = Inf(ini7:fin7);

% real market returns. Note returns are percentage
Ret_n    = Data1(:,2)+Data1(:,5);                  % nominal Return = risk premium + risk_free rate
Ret_n    = Ret_n(ini1:fin1);
Ret_r    = ((Ret_n/100+1)./Inf-1)*100;             % real returns=(1+Ret_n)/inflation -1 :Fisher equation
Mkt_mean = mean(Ret_r);                         % average market return
Mkt_std  = std(Ret_r);                          % standard deviation of market return
Mkt_ac   = AutoCorr(Ret_r);

% risk_free rate
Rf_n    = Data1(:,5);                              % nominal risk free rate
Rf_n    = Rf_n(ini1:fin1);                         % real risk free rate
Rf_r    = ((Rf_n/100+1)./Inf-1)*100;               % real returns=(1+Ret_n)/inflation -1 :Fisher equation
Rf_mean = mean(Rf_r);                            % average risk free rate
Rf_std  = std(Rf_r);                              % standard deviation of risk free rate
Rf_ac   = AutoCorr(Rf_r);

% risk premium
MktPrem     = Data1(:,2);                           % excess return
MktPrem     = MktPrem(ini1:fin1);
Mkt_Prem    = mean(MktPrem);
MktPrem_std = std(MktPrem);
MktPrem_ac  = AutoCorr(MktPrem);
SR          = Mkt_Prem/MktPrem_std;

% construct mkt volatility using monthly CRSP returns
MktPrem_M = Data19(ini19:fin19,2);
MktVol    = monthlyToAnnualVol(MktPrem_M,12)./100;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% print results 
fprintf('Asset Pricing Annual Moments(-99 represents NaN)\n');
Mean = [Mkt_mean Rf_mean Mkt_Prem SR];
Std  = [Mkt_std Rf_std MktPrem_std -99];
AC   = [Mkt_ac Rf_ac MktPrem_ac -99];
 
head = {'Mean';'Std';'AC'; };
row  = { 'Mkt'; 'RF'; 'Mkt Premium';'Sharpe Ratio'};
Forecasts = [Mean' Std' AC'];
texprint(Forecasts, '%6.2f', head, row);

% Macro quantities and credit spread
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GDP
GDP    = Data5(:,2);
GDPDef = Data6(:,2);
GDPGr  = GDP(ini5:fin5)./GDP(ini5-1:fin5-1)./(GDPDef(ini6:fin6)./GDPDef(ini6-1:fin6-1)) - 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Consumption
Con    = Data5(:,3);                                        % personal consumption expenditure(durable+nondurable+service)
ConGr  = Con(ini5:fin5)./Con(ini5-1:fin5-1)./((Cpi(ini7:fin7)./Cpi(ini7-1:fin7-1))) - 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compensation
Comp    = Data11(:,2);                                  % compensation of employees
LS      = Comp(ini11:fin11)./GDP(ini5:fin5)./1000;      % labor share
Emp     = Data9(:,2);                                   % employees
EmpGr   = Emp(ini9:fin9)./Emp(ini9-1:fin9-1) -1;        % employee growth

% wages and salaries per employee
Wage    = Data10(:,2);
WageGr  = Wage(ini10:fin10)./Wage(ini10-1:fin10-1)./(Cpi(ini7:fin7)./Cpi(ini7-1:fin7-1)) -1;

% investment
IGr    = (Data5(ini5:fin5,10)./Data6(ini6:fin6,10))./(Data5(ini5-1:fin5-1,10)./Data6(ini6-1:fin6-1,10)) -1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% corporate bond yields
Aaa     = Data4(ini4:fin4,3);                       % AAA corporate bond yield
Baa     = Data4(ini4:fin4,2);                       % BAA corporate bond yield
LY      = Data2(ini2:fin2,6);                       % Long-term 10 year governemnt bond yield
LYr     = ((1+LY./100)./Inf-1);                     % real long-term bond yield
Def     = Baa - LY;                                 % BAA premium
DefSpd  = Baa - Aaa;                                % BAA over AAA spread
SYr     = (Data2(ini2:fin2,8)-1);                   % real 1 year bond yield (it's real yield from Shiller)   
TSspd   = LY - Rf_n;                                % term spread 10 year government bond - 1 year government bond
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Default rate
DefRate = Data24(ini24:fin24,7); % Stops at 2006

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Shiller's Annual Data on S&P PE ratio
Ret_SP = Data2(ini2:fin2,16);       
PE_SP  = Data2(:,21);               
PE_SP  = (PE_SP(ini2:fin2));                        % S&P real price earnings ratio (PE10)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% financial leverage: non-financial corporate
% non-financial corporate book debt and market equity
FinLev = Data15(ini15:fin15,8)./(Data15(ini15:fin15,3)+Data15(ini15:fin15,8));                              % book debt/(mkt equity+book debt)
DGr    = (Data15(ini15:fin15,8)./(Data15(ini15-1:fin15-1,8))./(Cpi(ini7:fin7)./Cpi(ini7-1:fin7-1))) -1;     % debt growth

% Financial leverage flow measure using interests paid from NonFC corporate sector
InterestNonFC  = Data32(:,110);
IS             = InterestNonFC(ini32:fin32)./GDP(ini5:fin5);
TotalLev       = IS + LS;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% summary statistics
Var     = [GDPGr*100  WageGr*100 LS*100 IGr*100 DGr*100 PE_SP TSspd FinLev MktVol SYr*100 DefRate DefSpd EmpGr*100 ConGr*100 IS*100 TotalLev*100]; 
Var_std = std(Var);
Var_ac  = AutoCorr(Var);
XsCorr  = corrcoef(Var);
MVar    = mean(Var,1);

% 3.4 print results 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Print 
fprintf('Annual Moments Table A4\n');
head        = {'Mean';'Std';'Corr(dY,Var)';'AC';};
row         = { 'GDPGr';'WageGr';'LS';'IGr';'DebtGr';'P/E';'TermSpd';'FinLev';'MktVol';'SpotRate(1yr)';'DefRate';'DefSpd';'EmpGr';'ConGr';'IS';'TotalLev'};
Forecasts   = [MVar' Var_std'*1 XsCorr(1,:)'  Var_ac'];
texprint(Forecasts, ' %6.2f ', head, row);

%% Table 1 and Table 9: Predictive regression of Y_t+1 = a + b*X_t + e_t+1 
%  where X is either wage growth (WageGr) or labor share (LS) 
%  Y is either credit spread (DefSpd) or debt growth (DGr)
T         = length(DefSpd);
X         = ones(T,1);
dummy_dw  = 1; % 1 WageGr; 0 LS;
dummy_cs  = 1; % 1 DefSpd; 0 DGr

if dummy_dw == 1
   rhvR     = WageGr;    
   rhv5     = [X(1:end-1) rhvR(1:end-1) LS(1:end-1)];
else 
    rhvR    = LS;
    rhv5    = [X(1:end-1) rhvR(1:end-1) WageGr(1:end-1) ];

end

if dummy_cs ==1
    lhvR    = DefSpd(2:end);
    laglhvR = DefSpd(1:end);
else
    lhvR    = DGr(2:end)+1;
    laglhvR = DGr(1:end)+1;
end

rhv1    = [X(1:end-1) rhvR(1:end-1)];
rhv2    = [X(1:end-1) LS(1:end-1)];
rhv3    = [X(1:end-1) IS(1:end-1)];
rhv4    = [X(1:end-1) TotalLev(1:end-1)];
rhv6    = [X(1:end-1) rhvR(1:end-1) IGr(1:end-1)];
rhv7    = [X(1:end-1) rhvR(1:end-1) FinLev(1:end-1)];
rhv8    = [X(1:end-1) rhvR(1:end-1) MktVol(1:end-1)];
rhv9    = [X(1:end-1) rhvR(1:end-1) PE_SP(1:end-1)]; 
rhv10   = [X(1:end-1) rhvR(1:end-1) TSspd(1:end-1)./100]; 
rhv11   = [X(1:end-1) rhvR(1:end-1) SYr(1:end-1)];
rhv12   = [X(1:end-1) rhvR(1:end-1) GDPGr(1:end-1)]; 
rhv13   = [X(1:end-1) rhvR(1:end-1) ConGr(1:end-1)]; 
rhv14   = [X(1:end-1) rhvR(1:end-1) EmpGr(1:end-1)]; 
rhv15   = [X(1:end-1) rhvR(1:end-1) laglhvR(1:end-1)];
rhv16   = [X(1:end-1) LS(1:end-1) IS(1:end-1)]; 

lags   = sqrt(T); 
weight = 1; 

lhv = lhvR;
[bv1, sebv1, R2v1, R2vadj1] = olsgmm(lhv,rhv1,lags,weight);                
[bv2, sebv2, R2v2, R2vadj2] = olsgmm(lhv,rhv2,lags,weight);
[bv3, sebv3, R2v3, R2vadj3] = olsgmm(lhv,rhv3,lags,weight);
[bv4, sebv4, R2v4, R2vadj4] = olsgmm(lhv,rhv4,lags,weight); 
[bv5, sebv5, R2v5, R2vadj5] = olsgmm(lhv,rhv5,lags,weight);
[bv6, sebv6, R2v6, R2vadj6] = olsgmm(lhv,rhv6,lags,weight);
[bv7, sebv7, R2v7, R2vadj7] = olsgmm(lhv,rhv7,lags,weight);
[bv8, sebv8, R2v8, R2vadj8] = olsgmm(lhv,rhv8,lags,weight);
[bv9, sebv9, R2v9, R2vadj9] = olsgmm(lhv,rhv9,lags,weight); 
[bv10,sebv10,R2v10,R2vadj10]= olsgmm(lhv,rhv10,lags,weight); 
[bv11,sebv11,R2v11,R2vadj11]= olsgmm(lhv,rhv11,lags,weight); 
[bv12,sebv12,R2v12,R2vadj12]= olsgmm(lhv,rhv12,lags,weight); 
[bv13,sebv13,R2v13,R2vadj13]= olsgmm(lhv,rhv13,lags,weight); 
[bv14,sebv14,R2v14,R2vadj14]= olsgmm(lhv,rhv14,lags,weight); 
[bv15,sebv15,R2v15,R2vadj15]= olsgmm(lhv,rhv15,lags,weight); 
[bv16,sebv16,R2v16,R2vadj16]= olsgmm(lhv,rhv16,lags,weight); 
           
% print result for both Table 1 and Table 9 in the paper
disp(''); 
fprintf('Starting year: %6.0f\n',   year0);
fprintf('Ending year: %6.0f\n',     year1);

if dummy_cs ==1; disp('******Predicting credit spread (-99 represents NaN)******');
else; disp('******Predicting debt growth (-99 represents NaN)******');
end

if dummy_dw ==1
    disp('Regressions-WageGr'); 
    head = {'dW';'LS';'IGr';'FinLev';'MktVol';'PE';'TS';'RF';'GDPGr';'ConGr';'EmpGr';'lag y';'IS';'IS+LS';};
    row = { 'b(WGr)'; 'std(WGr)';'b(ctrl)'; 'std(ctrl)';'Adj R2'};
    moment=[bv1(2); sebv1(2);   -99;      -99;      R2vadj1;...
            bv5(2); sebv5(2);   bv5(3);  sebv5(3);  R2vadj5;   bv6(2);  sebv6(2);   bv6(3);  sebv6(3);  R2vadj6;...
            bv7(2); sebv7(2);   bv7(3);  sebv7(3);  R2vadj7;   bv8(2);  sebv8(2);   bv8(3);  sebv8(3);  R2vadj8;...
            bv9(2); sebv9(2);   bv9(3);  sebv9(3);  R2vadj9;   bv10(2); sebv10(2);  bv10(3); sebv10(3); R2vadj10;...
            bv11(2); sebv11(2); bv11(3); sebv11(3); R2vadj11;  bv12(2); sebv12(2);  bv12(3); sebv12(3); R2vadj12;...
            bv13(2); sebv13(2); bv13(3); sebv13(3); R2vadj13;  bv14(2); sebv14(2);  bv14(3); sebv14(3); R2vadj14;...
            bv15(2); sebv15(2); bv15(3); sebv15(3); R2vadj15;  -99;        -99;     bv3(2);  sebv3(2);  R2vadj3;...
            -99;        -99;     bv4(2);  sebv4(2); R2vadj4; ];
    texprint(reshape(moment,length(row),length(head)), ' %6.2f ', head, row);     
         
else
    disp('Regressions-LS'); 
    head = {'dW';'LS';'IGr';'FinLev';'MktVol';'PE';'TS';'RF';'GDPGr';'ConGr';'EmpGr';'lag y';'IS';'IS+LS';'IS';};
    row = { 'b(LS)'; 'std(LS)';'b(ctrl)'; 'std(ctrl)'; 'Adj R2'};
    
    moment=[bv5(2); sebv5(2);   bv5(3);  sebv5(3);  R2vadj5;   ...
            bv2(2);sebv2(2);   -99;     -99;       R2vadj2; bv6(2);  sebv6(2);   bv6(3);  sebv6(3);  R2vadj6;...
            bv7(2); sebv7(2);   bv7(3);  sebv7(3);  R2vadj7;   bv8(2);  sebv8(2);   bv8(3);  sebv8(3);  R2vadj8;...
            bv9(2); sebv9(2);   bv9(3);  sebv9(3);  R2vadj9;   bv10(2); sebv10(2);  bv10(3); sebv10(3); R2vadj10;...
            bv11(2); sebv11(2); bv11(3); sebv11(3); R2vadj11;  bv12(2); sebv12(2);  bv12(3); sebv12(3); R2vadj12;...
            bv13(2); sebv13(2); bv13(3); sebv13(3); R2vadj13;  bv14(2); sebv14(2);  bv14(3); sebv14(3); R2vadj14;...
            bv15(2); sebv15(2); bv15(3); sebv15(3); R2vadj15;  -99;        -99;     bv3(2);  sebv3(2);  R2vadj3;...
            -99;        -99;     bv4(2);  sebv4(2); R2vadj4;   bv16(2); sebv16(2);  bv16(3); sebv16(3); R2vadj16;  ];
        
   texprint(reshape(moment,length(row),length(head)), ' %6.2f ', head, row);     
end
  

%% Table 8: Predictive regression of WageGr_t+1 = a + b*X_t + e_t+1
T       = length(WageGr);
X       = ones(T,1);
rhv1    = [X(1:end-1) LS(1:end-1) FinLev(1:end-1)];
rhv2    = [X(1:end-1) DefSpd(1:end-1)./100];
lags    = sqrt(T); 
weight  = 1; 
lhv     = WageGr(2:end);
[bv1, sebv1, R2v1, R2vadj1, v1, F1] = olsgmm(lhv,rhv1,lags,weight);                
[bv2, sebv2, R2v2, R2vadj2, v2, F2] = olsgmm(lhv,rhv2,lags,weight);

% print results in Latex
disp(''); 
fprintf('Starting year: %6.0f\n',   year0);
fprintf('Ending year: %6.0f\n',     year1);

disp('******Predicting wage growth******');
head = {'WageGr'};
fmt  = ' %6.2f';
row  = { 'b(LS)'; 'std(LS)'; 'b(FinLev)'; 'std(FinLev)'; 'Adj R2'};
moment = [bv1(2,:);  sebv1(2,:);  bv1(3,:);  sebv1(3,:);  R2vadj1;]; 
texprint(moment, fmt, head, row);

row = { 'b(DefSpd)'; 'std(DefSpd)'; 'Adj R2'};
moment=[bv2(2,:);  sebv2(2,:);  R2vadj2;]; 
texprint(moment, fmt, head, row);

%% Table 10 Panel B

T         = length(DefSpd);
X         = ones(T,1);
[~,~,WageGr_R] = regress(WageGr,[X GDPGr]);
[~,~,LS_R]     = regress(LS,[X GDPGr]);

corr_WageGr_DefSpd = corr(WageGr_R(1:end-1),DefSpd(2:end));
corr_LS_DefSpd     = corr(LS_R(1:end-1),DefSpd(2:end));
corr_WageGr_DGr    = corr(WageGr_R(1:end-1),DGr(2:end));
corr_LS_DGr        = corr(LS_R(1:end-1),DGr(2:end));

disp('******Agg correlations******');
head = {'corr(WageGr_R,DefSpd)';'corr(LS_R,DefSpd)';'corr(WageGr_R,DGr)';'corr(LS_R,DGr)'};
fmt  = ' %6.2f';
row  = { 'correlation'};
moment = [corr_WageGr_DefSpd corr_LS_DefSpd  corr_WageGr_DGr  corr_LS_DGr]; 
texprint(moment, fmt, head, row);


%% Plot Figure A6 in the Online Appendix
if 0
    
load NBERDates.txt
% NBER recession dates
NBERan = NBERDates(:,4);
Dates  = NBERDates(:,1); iniN=find(Dates==year0); finN=find(Dates==year1); 
Dates  = Dates(iniN:finN);
NBERan = NBERan(iniN:finN);
fs     = 16; %fontsize

figure(1)
axes1 = axes('XMinorTick','on',  'YAxisLocation','right',  'YTick',[],  'XTick',[]);
axis([Dates(1) Dates(end) 0 1]);
box('on');
hold('all');

bar(Dates,NBERan,'BarWidth',1,'EdgeColor','none','FaceColor',[0.8 0.8 0.8]);
xlabel(' ','FontSize',fs)
h1 = gca;

%Create the second axes at the same location before plotting the second dataset.
h2 = axes('Position',get(h1,'Position'));
plot1 = plot([Dates,Dates,Dates],[zscore(DefSpd) zscore(WageGr) zscore(LS)]);
set(plot1(1),  'Color',[0 0 1],  'LineWidth',2.75);
set(plot1(2),  'Color',[1 0 0],  'LineWidth',2.75,'LineStyle', ':');
set(plot1(3),  'Color','k',  'LineWidth',2.75,'LineStyle', '-.');

% xlabel('Year','FontSize',fs)
legend({'CS','\DeltaW','LS'},'FontSize',fs); 
legend boxoff
set(h2,'YAxisLocation','left','Color','none','FontSize',fs-2)
set(h2,'XLim',get(h1,'XLim'),'Layer','top')
set(gcf,'PaperPositionMode','auto')

hold('off');
orient landscape 

% print -depsc2 C:\Users\
% print -dpdf  C:\Users\

end

