%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  
% This file does the time series predictability analysis at quarterly frequency
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clearvars; clc

%Here you pre-define the sample that you want to use
year0=1948; quarter0=year0*10+1; month0=year0*100+01; %Should start in first quarter.
year1=2014; quarter1=year1*10+4; month1=year1*100+12; %Should end in last quarter

% A description of data sets
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% quarterly data
% Data1: Shiller annual data:http://www.econ.yale.edu/~shiller/data.htm long term stock, bond, interest rate and consumption data 
% Data2: Bond yield: http://research.stlouisfed.org/fred2/series/BAA/downloaddata?cid=119. 
% Data3: NIPA table 1.1.5: contains nominal GDP
% Data4: NIPA table 1.1.9: contains implicit price of GDP
% Data6: BLSWage: contains the quarterly nonfarm labor data from BLS
% Data7: GSYld: contains treasury yields from the St Louis Fed
% Data9: monthly stock market volatility
% Data10: NIPATable2.1: contains quarterly personal income and its disposition
% Data11: Flow of Funds TableL103: contains nonfinancial corporate sector stock of asset and liabilities

% Load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Data1  = load('ShillerMonthly.txt');        ini1=find(Data1(:,1)==month0);       fin1=find(Data1(:,1)==month1); 
Data2  = load('MoodyBondYldMonthly.txt');   ini2=find(Data2(:,1)==month0);       fin2=find(Data2(:,1)==month1);                 
Data3  = load('NIPATable1.1.5.txt');        ini3=find(Data3(:,1)==quarter0);     fin3=find(Data3(:,1)==quarter1);
Data4  = load('NIPATable1.1.9.txt');        ini4=find(Data4(:,1)==quarter0);     fin4=find(Data4(:,1)==quarter1);
Data6  = load('BLSWage.txt');               ini6=find(Data6(:,1)==quarter0);     fin6=find(Data6(:,1)==quarter1);
Data7  = load('GSYld.txt');                 ini7=find(Data7(:,1)==month0);       fin7=find(Data7(:,1)==month1);
Data9  = load('monthly_vol_data.csv');      ini9=find(Data9(:,1)==month0);       fin9=find(Data9(:,1)==month1);
Data10 = load('NIPATable2.1.csv');          ini10=find(Data10(:,1)==quarter0);   fin10=find(Data10(:,1)==quarter1);
Data11 = load('TableL103.txt');             ini11=find(Data11(:,1)==quarter0);   fin11=find(Data11(:,1)==quarter1);

% Calculate financial data 
% Inflation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Quarter     = Data4(:,1);
Cpi         = Data4(:,3);                           % CPI index
Inf         = zeros(length(Cpi),1);
Inf(2:end)  = Cpi(2:end)./Cpi(1:end-1);             % inflation
Inf         = Inf(ini4:fin4);

% Macro quantities and credit spread
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GDP
GDP     = Data3(:,2);
GDPDef  = Data4(:,2);
GDPGr   = GDP(ini3:fin3)./GDP(ini3-1:fin3-1)./(GDPDef(ini4:fin4)./GDPDef(ini4-1:fin4-1)) - 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Consumption
Con    = Data3(:,3);                           % personal consumption expenditure(durable+nondurable+service)
ConGr  = Con(ini3:fin3)./Con(ini3-1:fin3-1)./((Cpi(ini4:fin4)./Cpi(ini4-1:fin4-1))) - 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compensation 
LS          = Data10(ini10:fin10,3)./Data3(ini3:fin3,2)/1000;   % NIPA personal compensation/GDP
CompGr      = Data10(ini10:fin10,3)./Data10(ini10-1:fin10-1,3)./(Cpi(ini4:fin4)./Cpi(ini4-1:fin4-1)) -1;

% Nonfarm business sector compensation per job 
Wage        = Data6(:,11)./Data6(:,7);           % compensation/No of jobs; 
WageGr      = Wage(ini6:fin6)./Wage(ini6-1:fin6-1)./(Cpi(ini4:fin4)./Cpi(ini4-1:fin4-1)) -1;

% Investment
IGr    = (Data3(ini3:fin3,10)./Data4(ini4:fin4,10))./(Data3(ini3-1:fin3-1,10)./Data4(ini4-1:fin4-1,10)) -1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bond yields
Aaa     = Data2(ini2:fin2,2);                       % AAA corporate bond yield
Baa     = Data2(ini2:fin2,3);                       % BAA corporate bond yield
DefSpdM = Baa - Aaa;
ind     = 0;                                        % 0: mean quarterly spread
DefSpd  = monthlyToQtr(DefSpdM,ind);                % average quarterly spread
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Treasury bond yields start in 1954 
% For TSspd as a control it needs starting in 1954
if year0>=1954
    LY      = Data7(ini7:fin7,3);                       % Long-term 10 year governemnt bond yield
    SY      = Data7(ini7:fin7,2);                       % Short-term 1 year governemnt bond yield
    TSM     = LY-SY;                                    % monthly term spread
    ind     = 0;                                        % 0: mean quarterly spread
    TSspd   = monthlyToQtr(TSM,ind);                    % average quarterly spread
    SYr     = ((1+monthlyToQtr(SY,ind)./100)./Inf-1);   % real long-term bond yield
else
    TSspd   = DefSpd;
    SYr     = DefSpd;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Shiller's monthly Data on S&P PE ratio
PE_SPM  = Data1(ini1:fin1,11);            % S&P real price earnings ratio (PE10)
ind     = 0;                              % 0: mean quarterly spread
PE_SP   = monthlyToQtr(PE_SPM,ind);       % average quarterly PE ratio

% volitility
Vol_M  = Data9(ini9:fin9,5);
MktVol = monthlyToQtr(Vol_M,ind);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% financial leverage: non-financial corporpoate business sector starts in 1952
Debt   = Data11(:,25)+Data11(:,29);                                                               % debt securities + loans
FinLev = Debt(ini11:fin11)./(Debt(ini11:fin11)+Data11(ini11:fin11,40));                           % book debt/(mkt equity+book debt)
DGr    = (Debt(ini11:fin11)./Debt(ini11-1:fin11-1))./(Cpi(ini4:fin4)./Cpi(ini4-1:fin4-1)) -1;     % debt growth


%% Table A6 and A7 Predictive regression : R_t+1 = a + b*WG_gr_t + e_t+1
% 1. note that TSspd and SYr start in 1954 thus regressions 7 and 8 need to
% change the starting year to 1954
% 2. note that DGr predictability regressions start in 1952 since quarterly
% debt starts in 1952

T         = length(DefSpd);
X         = ones(T,1);
lags      = 4;
weight    = 1; 

dummy_dw  = 1; % 1 CompGr; 2 WageGr (per job);0 LS; 
dummy_cs  = 1; % 1 DefSpd; 0 DGr
if dummy_dw     == 1
   rhvR  = CompGr; 
   rhv2    = [X(1:end-1) rhvR(1:end-1) LS(1:end-1)];
elseif dummy_dw == 0
   rhvR  = LS;
   rhv2    = [X(1:end-1)  LS(1:end-1)];
elseif dummy_dw == 2
   rhvR  = WageGr;
   rhv2    = [X(1:end-1) rhvR(1:end-1) LS(1:end-1)];
end

if dummy_cs ==1
    lhvR    = DefSpd(2:end);
    laglhvR = DefSpd(1:end);
else
    lhvR    = DGr(2:end)+1;
    laglhvR = DGr(1:end)+1;
end

rhv1    = [X(1:end-1) rhvR(1:end-1)];
rhv3    = [X(1:end-1) rhvR(1:end-1) IGr(1:end-1)];
rhv4    = [X(1:end-1) rhvR(1:end-1) FinLev(1:end-1)];
rhv5    = [X(1:end-1) rhvR(1:end-1) MktVol(1:end-1)];
rhv6    = [X(1:end-1) rhvR(1:end-1) PE_SP(1:end-1)]; 
rhv7    = [X(1:end-1) rhvR(1:end-1) TSspd(1:end-1)./100]; % Start in 1954
rhv8    = [X(1:end-1) rhvR(1:end-1) SYr(1:end-1)];        % Start in 1954
rhv9    = [X(1:end-1) rhvR(1:end-1) GDPGr(1:end-1)]; 
rhv10   = [X(1:end-1) rhvR(1:end-1) laglhvR(1:end-1)];
rhv11   = [X(1:end-1) rhvR(1:end-1) ConGr(1:end-1)]; 

lhv = lhvR;
[bv1, sebv1, R2v1, R2vadj1] = olsgmm(lhv,rhv1,lags,weight);                
[bv2, sebv2, R2v2, R2vadj2] = olsgmm(lhv,rhv2,lags,weight);
[bv3, sebv3, R2v3, R2vadj3] = olsgmm(lhv,rhv3,lags,weight);
[bv4, sebv4, R2v4, R2vadj4] = olsgmm(lhv,rhv4,lags,weight); 
[bv5, sebv5, R2v5, R2vadj5] = olsgmm(lhv,rhv5,lags,weight);
[bv6, sebv6, R2v6, R2vadj6] = olsgmm(lhv,rhv6,lags,weight);
[bv7, sebv7, R2v7, R2vadj7] = olsgmm(lhv,rhv7,lags,weight);
[bv8, sebv8, R2v8, R2vadj8] = olsgmm(lhv,rhv8,lags,weight);
[bv9, sebv9, R2v9, R2vadj9] = olsgmm(lhv,rhv9,lags,weight); 
[bv10,sebv10,R2v10,R2vadj10]= olsgmm(lhv,rhv10,lags,weight); 
[bv11,sebv11,R2v11,R2vadj11]= olsgmm(lhv,rhv11,lags,weight); 

% print results in Latex
disp(''); 
fprintf('Starting year: %6.0f\n',   year0);
fprintf('Ending year: %6.0f\n',     year1);

if dummy_cs ==1; disp('******Predicting credit spread******');else; disp('******Predicting debt growth******');end

if dummy_dw ==1
        disp('Regressions-CompGr');     
        head = {'dW';'LS';'IGr';'FinLev';'MktVol';'PE';'TS';'RF';'GDPGr';'lag y';'ConGr';};
        row = { 'b(CompGr)'; 'std(CompGr)';'b(ctrl)'; 'std(ctrl)';'Adj R2'};
        moment=[bv1(2); sebv1(2);      -99;      -99;   R2vadj1;    bv2(2); sebv2(2);   bv2(3);  sebv2(3);  R2vadj2;...  
                bv3(2); sebv3(2);   bv3(3);  sebv3(3);  R2vadj3;    bv4(2); sebv4(2);   bv4(3);  sebv4(3);  R2vadj4;...              
                bv5(2); sebv5(2);   bv5(3);  sebv5(3);  R2vadj5;    bv6(2); sebv6(2);   bv6(3);  sebv6(3);  R2vadj6;...
                bv7(2); sebv7(2);   bv7(3);  sebv7(3);  R2vadj7;    bv8(2); sebv8(2);   bv8(3);  sebv8(3);  R2vadj8;...
                bv9(2); sebv9(2);   bv9(3);  sebv9(3);  R2vadj9;    bv10(2);sebv10(2);  bv10(3); sebv10(3); R2vadj10;...
                bv11(2); sebv11(2); bv11(3); sebv11(3); R2vadj11; ];
        texprint(reshape(moment,length(row),length(head)), ' %6.2f ', head, row);     
        
elseif dummy_dw == 2
        disp('Regressions-WageGr (per job)'); 
        
        head = {'dW';'LS';'IGr';'FinLev';'MktVol';'PE';'TS';'RF';'GDPGr';'lag y';'ConGr';};
        row = { 'b(WGr)'; 'std(WGr)';'b(ctrl)'; 'std(ctrl)';'Adj R2'};
        moment=[bv1(2); sebv1(2);      -99;      -99;   R2vadj1;    bv2(2); sebv2(2);   bv2(3);  sebv2(3);  R2vadj2;...  
                bv3(2); sebv3(2);   bv3(3);  sebv3(3);  R2vadj3;    bv4(2); sebv4(2);   bv4(3);  sebv4(3);  R2vadj4;...              
                bv5(2); sebv5(2);   bv5(3);  sebv5(3);  R2vadj5;    bv6(2); sebv6(2);   bv6(3);  sebv6(3);  R2vadj6;...
                bv7(2); sebv7(2);   bv7(3);  sebv7(3);  R2vadj7;    bv8(2); sebv8(2);   bv8(3);  sebv8(3);  R2vadj8;...
                bv9(2); sebv9(2);   bv9(3);  sebv9(3);  R2vadj9;    bv10(2);sebv10(2);  bv10(3); sebv10(3); R2vadj10;...
                bv11(2); sebv11(2); bv11(3); sebv11(3); R2vadj11; ];
        texprint(reshape(moment,length(row),length(head)), ' %6.2f ', head, row);     

elseif  dummy_dw == 0
        disp('Regressions-LS'); 
        head = {'LS';'IGr';'FinLev';'MktVol';'PE';'TS';'RF';'GDPGr';'lag y';'ConGr';};
        row = { 'b(LS)'; 'std(LS)';'b(ctrl)'; 'std(ctrl)';'Adj R2'};
        moment=[bv1(2); sebv1(2);      -99;      -99;   R2vadj1; ...  
                bv3(2); sebv3(2);   bv3(3);  sebv3(3); R2vadj3;     bv4(2);  sebv4(2);   bv4(3);  sebv4(3);  R2vadj4;...              
                bv5(2); sebv5(2);   bv5(3);  sebv5(3);  R2vadj5;    bv6(2);  sebv6(2);   bv6(3);  sebv6(3);  R2vadj6;...
                bv7(2); sebv7(2);   bv7(3);  sebv7(3);  R2vadj7;    bv8(2);  sebv8(2);   bv8(3);  sebv8(3);  R2vadj8;...
                bv9(2); sebv9(2);   bv9(3);  sebv9(3);  R2vadj9;    bv10(2); sebv10(2);  bv10(3); sebv10(3); R2vadj10;...
                bv11(2); sebv11(2); bv11(3); sebv11(3); R2vadj11; ];
        texprint(reshape(moment,length(row),length(head)), ' %6.2f ', head, row);     
end


